@@ -1,7 +1,7 @@
 use std::path::{Path, PathBuf};
 use crate::smol::{AsyncMmapFileExt, AsyncMmapFileMutExt, AsyncMmapFileReader, AsyncMmapFileWriter};
 use crate::MetaData;
-use crate::error::{Error, Result};
+use crate::error::{Error, ErrorKind, Result};
 use crate::metadata::EmptyMetaData;
 
 declare_and_impl_async_empty_mmap_file!();
