@@ -4,53 +4,40 @@ use bytes::{Bytes, BytesMut};
 use crate::{MmapFileExt, MmapFileMutExt, MetaData};
 use crate::metadata::MemoryMetaData;
 
-pub struct MemoryMmapFile {
-    mmap: Bytes,
-    path: PathBuf,
-    create_at: SystemTime,
-}
+define_impl_constructor_for_mmap_file!(MemoryMmapFile, "MemoryMmapFile", "");
 
 impl_mmap_file_ext!(MemoryMmapFile);
 
-pub struct MemoryMmapFileMut {
-    mmap: BytesMut,
-    path: PathBuf,
-    create_at: SystemTime,
-}
+define_and_impl_constructor_for_mmap_file_mut!(MemoryMmapFileMut, "MemoryMmapFileMut", MemoryMmapFile, "MemoryMmapFile", "MmapFileExt", "");
 
 impl_mmap_file_ext!(MemoryMmapFileMut);
 
 impl MmapFileMutExt for MemoryMmapFileMut {
+    #[inline]
     fn as_mut_slice(&mut self) -> &mut [u8] {
         self.mmap.as_mut()
     }
 
-    fn flush(&self) -> crate::error::Result<()> {
-        Ok(())
+    #[inline]
+    fn is_cow(&self) -> bool {
+        false
     }
 
-    fn flush_async(&self) -> crate::error::Result<()> {
-        Ok(())
-    }
-
-    fn flush_range(&self, _offset: usize, _len: usize) -> crate::error::Result<()> {
-        Ok(())
-    }
-
-    fn flush_async_range(&self, _offset: usize, _len: usize) -> crate::error::Result<()> {
-        Ok(())
-    }
+    noop_flush!();
 
+    #[inline]
     fn truncate(&mut self, max_sz: u64) -> crate::error::Result<()> {
         self.mmap.resize(max_sz as usize, 0);
         Ok(())
     }
 
-    fn delete(self) -> crate::error::Result<()> {
+    #[inline]
+    fn remove(self) -> crate::error::Result<()> {
         Ok(())
     }
 
+    #[inline]
     fn close_with_truncate(self, _max_sz: i64) -> crate::error::Result<()> {
         Ok(())
     }
-}
+}
\ No newline at end of file
